//
//  System.m
//  SolSystem
//
//  Created by Paul Jungwirth on 11/14/09.
//  Copyright 2009 Illuminated Computing. All rights reserved.
//

#import "System.h"
#import "Planet.h"

static float G = 6.67e-20;	// km^3 * kg^-1 * s^-2

@implementation System

- (id) init {
	Planet* p;
	self = [super init];
	if (self) {
		_lock = [[NSLock alloc] init];
		_kmHeight = 10;
		_kmWidth = 10;
		_planets = [[NSMutableArray alloc] init];

		p = [[Planet alloc] init];
		[p setName:@"Sun"];
		[p setKmX:0];
		[p setKmY:0];
		[p setVelX:0];
		[p setVelY:0];
		[p setMass:1.98892e30];
		[p setRadius:695500];
		[p setColor:[NSColor yellowColor]];
		[_planets addObject:p];

		p = [[Planet alloc] init];
		[p setName:@"Earth"];
		[p setKmX:1.495978707e8];	// km
		[p setKmY:0];
		[p setVelX:0];
		[p setVelY:29.8];	// km/s
		[p setMass:5.9742e24];	// kg
		[p setRadius:6378.1];	// km
		[p setColor:[NSColor blueColor]];
		[_planets addObject:p];

		p = [[Planet alloc] init];
		[p setName:@"Mercury"];
		[p setKmX:5.79091e7];	// km
		[p setKmY:0];
		[p setVelX:0];
		[p setVelY:47.87];	// km/s
		[p setMass:3.3022e23];
		[p setRadius:2439.7];
		[p setColor:[NSColor redColor]];
		[_planets addObject:p];
		
		p = [[Planet alloc] init];
		[p setName:@"Venus"];
		[p setKmX:1.08208930e8];	// km
		[p setKmY:0];
		[p setVelX:0];
		[p setVelY:35.02];	// km/s
		[p setMass:4.8685e24];
		[p setRadius:6051.8];
		[p setColor:[NSColor purpleColor]];
		[_planets addObject:p];
		
		p = [[Planet alloc] init];
		[p setName:@"Moon"];
		[p setKmX:1.499822697e8];	// km
		[p setKmY:0];
		[p setVelX:0];
		[p setVelY:30.822];	// km/s
		[p setMass:7.3477e22];
		[p setRadius:1737.10];
		[p setColor:[NSColor lightGrayColor]];
		[_planets addObject:p];
	}
	return self;
}

- (void) advanceTime:(float) secs {
	NSEnumerator* it = [_planets objectEnumerator];
	Planet *p;
	float dx, dy, ddx, ddy;
	while (p = (Planet *)[it nextObject]) {
		// First move the object, then change its velocity based on acceleration from other planets:
		
		dx = [p velX];
		dy = [p velY];
//		NSLog(@"%@'s velocity is %f", [p name], sqrt(pow(dx,2) + pow(dy,2)));
		
		// Now for the current planet p, compute the acceleration upon it caused by all the other planets:
		
		NSEnumerator* jt = [_planets objectEnumerator];
		Planet *q;
		ddx = ddy = 0;
		while (q = (Planet *)[jt nextObject]) {
			if (q != p) {
				Vector d = [p distanceFrom:q];
				if (d.magnitude > 0) {
					float a = G * [q mass] / pow(d.magnitude, 2);
					// subtract, not add, because gravity attracts:
					ddx -= d.i * a / d.magnitude;
					ddy -= d.j * a / d.magnitude;
				} else {
					NSLog(@"ERROR: d.magnitude is not greater than zero!");
					// Note that this is not really an error if two planets crash into each other.
					// But then I guess I should just ignore their attraction to each other,
					// and they will forever travel as one body.
				}
			}
		}
//		NSLog(@"%@'s acceleration is %f, %f", [p name], ddx, ddy);
		
		[p setVelX:[p velX] + ddx * secs];
		[p setVelY:[p velY] + ddy * secs];
		[p setKmX:[p kmX] + dx * secs];
		[p setKmY:[p kmY] + dy * secs];
	}
}

- (float) maxPos {
	float max = 0;
	NSEnumerator* it = [_planets objectEnumerator];
	Planet* p;
	while (p = (Planet *)[it nextObject]) {
		if ([p kmX] > max) max = [p kmX];
		if ([p kmY] > max) max = [p kmY];
	}	
	return max;
}

- (void) dealloc {
	[_planets release];
	[super dealloc];
}

- (NSArray*) planets { return _planets; }

- (float) kmHeight { return _kmHeight; }
- (void) setKmHeight:(float) h { _kmHeight = h; }

- (float) kmWidth { return _kmWidth; }
- (void) setKmWidth:(float) w { _kmWidth = w; }

- (void) moveTime {  }

- (void) lock {
	[_lock lock];
}

- (void) unlock {
	[_lock unlock];
}


@end
